package com.atm.bondowowo.ui.login

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.widget.Button
import android.widget.EditText
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.Observer
import com.atm.bondowowo.MainActivity
import com.atm.bondowowo.R
import com.atm.bondowowo.data.model.LoginRequest
import com.atm.bondowowo.utils.DialogHelper
import com.atm.bondowowo.utils.NetworkStateLiveData
import com.atm.bondowowo.utils.NetworkUtils.apiService
import com.atm.bondowowo.utils.UserPreferencesUtil
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class LoginActivity : AppCompatActivity() {

    private lateinit var etUsername: EditText
    private lateinit var etPassword: EditText
    private lateinit var btnLogin: Button

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_login)

        initViews()
        setupNetworkStateObserver()
        checkAuthenticationStatus()
    }

    private fun initViews() {
        etUsername = findViewById(R.id.etUsername)
        etPassword = findViewById(R.id.etPassword)
        btnLogin = findViewById(R.id.btnLogin)
        btnLogin.setOnClickListener { performLogin(this) }
    }

    private fun setupNetworkStateObserver() {
        val networkStateLiveData = NetworkStateLiveData(this)
        networkStateLiveData.observe(this, Observer { isConnected ->
            if (!isConnected) {
                DialogHelper.showNoInternetDialog(this)
            }
        })
    }

    private fun checkAuthenticationStatus() {
        CoroutineScope(Dispatchers.IO).launch {
            try {
                val isAuthenticated = isUserAuthenticated()
                if (isAuthenticated) {
                    navigateToMainActivity()
                }
            } catch (e: Exception) {
                withContext(Dispatchers.Main) {
                    Toast.makeText(
                        this@LoginActivity,
                        "You might have unstable connection",
                        Toast.LENGTH_SHORT
                    )
                        .show()
                }
            }
        }
    }

    private fun performLogin(context: Context) {
        val email = etUsername.text.toString()
        val password = etPassword.text.toString()

        val loginRequest = LoginRequest(email, password)

        CoroutineScope(Dispatchers.IO).launch {
            try {
                val response = apiService.login(loginRequest)
                if (response.isSuccessful) {
                    val loginResponse = response.body()
                    val token = loginResponse?.token

                    if (token != null) {
                        UserPreferencesUtil.addJWT(context, token)
                        UserPreferencesUtil.addEmail(context, email)
                    }

                    startActivity(Intent(this@LoginActivity, MainActivity::class.java))
                    finish()
                } else {
                    withContext(Dispatchers.Main) {
                        Toast.makeText(
                            this@LoginActivity,
                            "Invalid Credentials",
                            Toast.LENGTH_SHORT
                        )
                            .show()
                    }
                }
            } catch (e: Exception) {
                withContext(Dispatchers.Main) {
                    Toast.makeText(this@LoginActivity, "An error occurred", Toast.LENGTH_SHORT)
                        .show()
                }
            }
        }
    }

    private suspend fun isUserAuthenticated(): Boolean {
        val sharedPreferences = getSharedPreferences("AUTH_PREFS", MODE_PRIVATE)
        val token = sharedPreferences.getString("JWT_TOKEN", null)
        val isValid = token?.let { isTokenValid(it) }

        return token != null && isValid == true
    }

    private fun navigateToMainActivity() {
        startActivity(Intent(this, MainActivity::class.java))
        finish()
    }

    private suspend fun isTokenValid(token: String): Boolean {
        // Karena lagi malas login
//        return true
        return try {
            val response = apiService.verifyToken("Bearer $token")
            if (response.isSuccessful) {
                val responseBody = response.body()
                val isString = responseBody is String
                return !isString
            } else {
                false
            }
        } catch (e: Exception) {
            false
        }
    }
}