package com.atm.bondowowo.ui.twibbon

import android.os.Bundle
import androidx.appcompat.app.AppCompatActivity
import com.atm.bondowowo.databinding.ActivityTwibbonBinding
import android.Manifest
import android.content.Context
import android.content.pm.PackageManager
import android.util.Log
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.activity.result.contract.ActivityResultContracts
import androidx.camera.core.CameraSelector
import androidx.camera.core.Preview
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.camera.view.PreviewView
import androidx.core.content.ContextCompat
import com.atm.bondowowo.R
import com.google.common.util.concurrent.ListenableFuture


class TwibbonActivity : AppCompatActivity() {

    private lateinit var binding: ActivityTwibbonBinding

    private lateinit var mContext: Context

    private lateinit var previewView: PreviewView
    private lateinit var previewImage: ImageView
    private lateinit var twibbon: ImageView
    private lateinit var cameraProviderFuture: ListenableFuture<ProcessCameraProvider>
    private lateinit var warningText: TextView
    private lateinit var capturePhotoButton: Button
    private var imagePreview: Boolean = false

    private val requestPermissionLauncher =
        registerForActivityResult(ActivityResultContracts.RequestPermission()) { isGranted: Boolean ->
            if (isGranted) {
                warningText.visibility = View.GONE
                capturePhotoButton.visibility = View.VISIBLE
                previewView.visibility = View.VISIBLE
                twibbon.visibility = View.VISIBLE
                Log.d("Twibbon", "berhasil granted")
            } else {
                warningText.visibility = View.VISIBLE
                warningText.text = resources.getString(R.string.twibbon_warning_no_permissions)
                capturePhotoButton.visibility = View.GONE
                previewView.visibility = View.GONE
                twibbon.visibility = View.GONE
                // Request permission again
                requestCameraPermission()
                Log.e("Twibbon", "GAGAL GRANTED")
            }
        }

    private fun requestCameraPermission() {
        requestPermissionLauncher.launch(Manifest.permission.CAMERA)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityTwibbonBinding.inflate(layoutInflater)
        setContentView(binding.root)

        mContext = this
        cameraProviderFuture = ProcessCameraProvider.getInstance(mContext)

        previewView = findViewById(R.id.previewView)
        previewView.implementationMode = PreviewView.ImplementationMode.PERFORMANCE
        previewView.scaleType = PreviewView.ScaleType.FILL_CENTER
        previewView.visibility = View.VISIBLE

        previewImage = findViewById(R.id.imagePreview)
        twibbon = findViewById(R.id.twibbon)
        twibbon.visibility = View.VISIBLE

        warningText = findViewById(R.id.warningCameraTwibbon)
        warningText.visibility = View.GONE

        capturePhotoButton = findViewById(R.id.btnCapture)
        capturePhotoButton.visibility = View.VISIBLE

        capturePhotoButton.setOnClickListener {
            if (imagePreview) {
                previewView.visibility = View.VISIBLE
                previewImage.visibility = View.GONE
                capturePhotoButton.text = resources.getString(R.string.btn_capture)
                imagePreview = false
            } else {
                previewView.visibility = View.GONE
                previewImage.visibility = View.VISIBLE
                capturePhotoButton.text = resources.getString(R.string.btn_take_photo)
                imagePreview = true
                val bitmap = previewView.bitmap
                previewImage.setImageBitmap(bitmap)
                Toast.makeText(mContext, "Image captured", Toast.LENGTH_SHORT).show()
            }
        }
    }

    override fun onStart() {
        super.onStart()
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA) != PackageManager.PERMISSION_GRANTED) {
            requestCameraPermission()
        } else {
            cameraProviderFuture.addListener({
                val cameraProvider = cameraProviderFuture.get()
                bindPreview(cameraProvider)
            }, ContextCompat.getMainExecutor(mContext))
        }
    }


    override fun onDestroy() {
        super.onDestroy()
        cameraProviderFuture.get().unbindAll()
    }

    private fun bindPreview(cameraProvider: ProcessCameraProvider) {
        val preview: Preview = Preview.Builder().build()

        val cameraSelector: CameraSelector = CameraSelector.Builder()
            .requireLensFacing(CameraSelector.LENS_FACING_BACK)
            .build()

        preview.setSurfaceProvider(previewView.surfaceProvider)

        cameraProvider.bindToLifecycle(this, cameraSelector, preview)
    }

}